﻿import mvcclock.ClockModel;
import mvc.*;
import util.*;

/**
 * Makes changes to the ClockModel's data based on user input.
 * Provides general services that any view might find handy.
 */
class mvcclock.ClockController extends AbstractController {

  /**
   * Constructor
   *
   * @param   cm   The model to modify.
   */
  public function ClockController (cm:Observable) {
    super(cm);
  }

  /**
   * Starts the clock ticking.
   */
  public function startClock ():Void {
    ClockModel(getModel()).start();
  }

  /**
   * Stops the clock ticking.
   */
  public function stopClock ():Void {
    ClockModel(getModel()).stop();
  }

  /**
   * Resets the clock's time to 12 midnight.
   */
  public function resetClock ():Void {
    setTime(0, 0, 0);
  }

  /**
   * Changes the clock's time.
   *
   * @param   h   The new hour.
   * @param   m   The new minute.
   * @param   s   The new second.
   */
  public function setTime (h:Number, m:Number, s:Number):Void {
    ClockModel(getModel()).setTime(h, m, s);
  }

  // As these next three methods show, the controller can provide
  // convenience methods to change data in the model.

  /**
   * Sets just the clock's hour.
   *
   * @param   h   The new hour.
   */
  public function setHour (h:Number):Void {
    ClockModel(getModel()).setTime(h, null, null);
  }

  /**
   * Sets just the clock's minute.
   *
   * @param   m   The new minute.
   */
  public function setMinute (m:Number):Void {
    ClockModel(getModel()).setTime(null, m, null);
  }

  /**
   * Sets just the clock's second.
   *
   * @param   s   The new second.
   */
  public function setSecond (s:Number):Void {
    ClockModel(getModel()).setTime(null, null, s);
  }

  /**
   * Handles events from the start, stop, and reset buttons
   * of the ClockTools view.
   */
  public function click (e:Object):Void {
    switch (e.target._name) {
      case "start":
        startClock();
        break;
      case "stop":
        stopClock();
        break;
      case "reset":
        resetClock();      
        break;
    }
  }
}